;;; jxw.el - Gichitaro module for GNUemacs/NEmacs
;;; Copyright (C) HASHIMOTO Ryoichi
;;;        Industrial Products Research Institute, AIST/MITI Japan
;;;        Higashi 1-1-4, Tsukuba-shi, IBARAKI-ken, Zip(305), Japan
;;;        E-mail: hashimoto@ipri.go.jp
;;;        Tel: 0298-54-6661 (Desk)
;;;        Fax: 0298-54-6608

;;; This module provides features to emulate jxw-ichitaro's menu
;;; oriented man-machine interface in GNUemacs/NEmacs.

;;; This is not an officially recognized part of GNU emacs nor NEmacs.
;;; But this must be used, copied, distributed, or modified under the
;;; same principle which GNU emacs and NEmacs assert.  

;;; You may freely copy, distribute, or modify this module, provided
;;; that: 

;;; 1) this copyrights note and comments are included without any 
;;;   modification; (you may add your comments, that's no problem.)  
;;; 2) you keep books, when you modify thid module, on the version 
;;;    history listed at the bottom of this comment field. In such a
;;;    case, add your initials to the version number so that cousin
;;;    versions can be easily distinguished.

;;; Do feel free to write to the author either to encourage him or to 
;;; complain anything.

;;; BUGS: 
;;; * Currently, it is up to you to install ATOK-like key bindings for
;;;   WNN, though author does not admit it a bug.
;;; * Some users also complained that many of the menu items are not
;;;   supported, such as formatting, table frameworks, printings, etc.
;;;   Author has a project to install formatting and calc features.  
;;;   Do help him by sharing the load, or wait patiently!

;;; Version history:
;;;    alpha-1 1989-Jul-30 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, Japan
;;;       jxw-root-menu, jxw-delete-submenu, jxw-move-submenu
;;;       jxw-copy-submenu
;;;    alpha-2 1989-Aug-12 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, Japan
;;;       jxw-trans-submenu
;;;    alpha-3 1989-Sep-12 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, Japan
;;;       completely changed to use popup-menu.el as menu interface.
;;;    alpha-4 1989-Sep-15 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, Japan
;;;       jxw-SR-submenu jxw-jump-submenu jxw-window-submenu
;;;    alpha-5 1989-Oct-14 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, Japan
;;;       jxw-option-submenu
;;;    1.00RH 1989-Nov-10 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, japan
;;;       Copyrights note and comments are added.
;;;    1.10RH 1990-Feb-25 by HASHIMOTO Ryoichi, IPRI/AIST/MITI, japan
;;;       Receive- and send-mail is installed in root menu.
;;;       Bug in jxw-trans is fixed.
;;;       Jxw-options is changed to display sub menu and to allow the 
;;;       site to add local features.
;;;       Rub-out char on i-search is changed from ^? to \C-h.

(require 'popup-menu)
(provide 'jxw-root)

(global-set-key "\e\e" 'jxw-popup-root-menu)
(global-set-key "\e\\" 'eval-expression)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun jxw-erase-uncommon-del-keys()
    (local-unset-key "\177")
    (local-unset-key "\C-h")
    (local-unset-key "\e\177")
    (local-unset-key "\e\C-h")
)

(setq c-mode-hook
      '(lambda ()
	 (jxw-erase-uncommon-del-keys)
	)
)

(setq emacs-lisp-mode-hook
    '(lambda ()
       (jxw-erase-uncommon-del-keys)
     )
)

(let ((*buffer-name* (buffer-name)))
      (switch-to-buffer "*scratch*")
      (fundamental-mode)
      (switch-to-buffer *buffer-name*)
)

(global-set-key "\177" 'delete-char)
(global-set-key "\C-h" 'delete-backward-char)
(global-set-key "\e\177" 'kill-word)
(global-set-key "\e\C-h" 'backward-kill-word)
(setq search-delete-char ?\C-h)

(define-key minibuffer-local-completion-map "\C-n" 'scroll-other-window)
(define-key minibuffer-local-completion-map "\e\e" 'abort-recursive-edit)
(define-key minibuffer-local-map "\e\e" 'abort-recursive-edit)
(define-key minibuffer-local-must-match-map "\e\e" 'abort-recursive-edit)
(define-key minibuffer-local-ns-map "\e\e" 'abort-recursive-edit)

(defvar jxw-beginning-prompt t)
(defvar jxw-execution-prompt t)

(defvar jxw-root-menu-set nil)
(setq jxw-root-menu-set (make-menu-set "*root menu*" 10))
(menu-define-key jxw-root-menu-set "a" 'what-line      "ʸ  ")
(menu-define-key jxw-root-menu-set "t" 'jxw-trans-submenu  "ե  ")
(menu-define-key jxw-root-menu-set "d" 'jxw-delete-submenu "      ")
(menu-define-key jxw-root-menu-set "m" 'jxw-move-submenu   "ư      ")
(menu-define-key jxw-root-menu-set "c" 'jxw-copy-submenu   "ԡ    ")
(menu-define-key jxw-root-menu-set "y" 'yank               "ڡ  ")
(menu-define-key jxw-root-menu-set "b" 'jxw-not-necessary  "ꥢ    ")
(menu-define-key jxw-root-menu-set "p" 'jxw-not-necessary  "      ")
(menu-define-key jxw-root-menu-set "s" 'jxw-SR-menu        "      ")
(menu-define-key jxw-root-menu-set "j" 'jxw-jump-submenu  "  ")
(menu-define-key jxw-root-menu-set "f" 'jxw-formatter-submenu "  ")
(menu-define-key jxw-root-menu-set "e" 'jxw-not-necessary  "ʸ  ")
(menu-define-key jxw-root-menu-set "z" 'jxw-not-necessary  "ʸ  ")
(menu-define-key jxw-root-menu-set "h" 'jxw-not-necessary  "Ͽ  ")
(menu-define-key jxw-root-menu-set "w" 'jxw-window-submenu "ɥ")
(menu-define-key jxw-root-menu-set "k" 'jxw-not-necessary  "      ")
(menu-define-key jxw-root-menu-set "l" 'jxw-not-necessary  "Ѵ  ")
(menu-define-key jxw-root-menu-set "v" 'jxw-not-necessary  "׻      ")
(menu-define-key jxw-root-menu-set "x" 'jxw-not-necessary  "ȹ    ")
(menu-define-key jxw-root-menu-set "o" 'jxw-options-submenu  "ץ")
(menu-define-key jxw-root-menu-set "q" 'save-buffers-kill-emacs "λ      ")
(menu-define-key jxw-root-menu-set "r" 'jxw-not-necessary  "å    ")
(menu-define-key jxw-root-menu-set "g" 'rmail "͹ؼ  ")
(menu-define-key jxw-root-menu-set "i" 'mail  "͹  ")

(defun jxw-popup-root-menu ()
  "displays root-menu for jxw"
  (interactive)
  (menu-display-menu jxw-root-menu-set)
)

(defun jxw-not-necessary ()
  " "
  (interactive)
  (message "εǽ UNIX ǤפǤ")
)

;;; End of jxw-root-menu
;;; jxw-SR - features for search and replace.

(defvar jxw-SR-menu-set nil "")
(defun jxw-SR-menu ()
  "Displays Search and Replace manu"
  (interactive)
  (if jxw-SR-menu-set
      ()
    (setq jxw-SR-menu-set (make-menu-set "*/ִ*" 18))
    (menu-define-key jxw-SR-menu-set "s" 're-search-forward "ʸ      ")
    (menu-define-key jxw-SR-menu-set "z" 're-search-backward "ʸƬ      ")
    (menu-define-key jxw-SR-menu-set "q" 'query-replace-regexp "ʸִ(ǧ)")
    (menu-define-key jxw-SR-menu-set "p" 'jxw-query-replace-backward "ʸƬִ(ǧ)")
    (menu-define-key jxw-SR-menu-set "a" 'replace-regexp "ʸִ(쵤)")
    (menu-define-key jxw-SR-menu-set "b" 'jxw-replace-regexp-backward "ʸƬִ(쵤)")
  )
  (menu-display-menu jxw-SR-menu-set)
)

(defun jxw-query-replace-backward (old new)
  "Replaces regular expression backwards"
  (interactive "sִ: \nsִ: ")
  (let ((tmp-char 0) (jxw-continue t))
    (while (and jxw-continue (re-search-backward old (point-min) t))
      (while (progn
	       (message "ڡ:ִ   DEL:̵   ESC:")
	       (setq tmp-char (read-char))
	       (null (memq tmp-char '(?  ?\177 ?\033)))
	     )
      )
      (cond
        ((= tmp-char ? )
	 (replace-match new)
	)
	((= tmp-char ?\033)
	 (setq jxw-continue nil)
	)
      )
    )
  )
)

(defun jxw-replace-regexp-backward (old new)
  "Replaces regular expression backwards"
  (interactive "sִ: \nsִ: ")
  (let ((tmp-point nil))
    (while (re-search-backward old (point-min) t)
      (setq tmp-point (point))
      (replace-match new)
      (goto-char tmp-point)
    )
  )
)

;;; End of jxw-SR
;;;  jxw-dmc - features for Delete, Move, and Copy

(require 'popup-menu)

(defvar *jxw-dmc-flag* 0)
(defun jxw-delete-submenu ()
  "line- characer- or buffer-wise deletion"
  (interactive)
  (setq *jxw-dmc-flag* 0)
  (menu-display-menu jxw-delete-menu-set)
)

(defun jxw-move-submenu ()
   "Moves string line- character- or block-wise"
  (interactive)
  (setq *jxw-dmc-flag* 1)
  (menu-display-menu jxw-mc-menu-set)
)
(defun jxw-copy-submenu ( )
 "copies line- character- and block-wise"
  (interactive)
  (setq *jxw-dmc-flag* 2)
  (menu-display-menu jxw-mc-menu-set)
)

(defvar jxw-delete-menu-set nil "menu-set for delete")
(if jxw-delete-menu-set
    ()
  (setq jxw-delete-menu-set (make-menu-set "*delete*" 10))
  (menu-define-key jxw-delete-menu-set "l" 'jxw-dmc-linewise "ñ    ")
  (menu-define-key jxw-delete-menu-set "c" 'jxw-dmc-charwise "ʸñ  ")
  (menu-define-key jxw-delete-menu-set "a" 'jxw-delete-bufferwise "ʸ񤴤  ")
)

(defvar jxw-mc-menu-set nil "menu-set for move and copy")
(if jxw-mc-menu-set
    ()
  (setq jxw-mc-menu-set (make-menu-set "*move/copy*" 10))
  (menu-define-key jxw-mc-menu-set "l" 'jxw-dmc-linewise "ñ    ")
  (menu-define-key jxw-mc-menu-set "c" 'jxw-dmc-charwise "ʸñ  ")
  (menu-define-key jxw-mc-menu-set "b" 'jxw-mc-blockwise "blockñ ")
)

(defun jxw-dmc-linewise ()
  "Deletes linewise"
  (interactive)
  (setq *jxw-dmc-wise* 1)
  (jxw-dmc-wise)
)

(defun jxw-dmc-charwise ()
    "deletes character-wise"
   (interactive)
   (setq *jxw-dmc-wise* 0)
   (jxw-dmc-wise)
)

(defun jxw-mc-blockwise ()
    "move or copy block-wise"
   (interactive)
   (setq *jxw-dmc-wise* 2)
   (jxw-dmc-wise)
)

(defun jxw-delete-bufferwise ( )
  "Deletes all the contents of the buffer."
  (interactive)
  (kill-region (point-min) (point-max))
  (message "Ƥƺޤ")
)

(defvar *jxw-delete-point* 0)
(defvar *jxw-delete-linewise-point* 0)

(defvar *jxw-dmc-wise* 0)

(defun jxw-dmc-wise ()
  "Undertaking process to delete, move, or copy"
  (interactive)
  (if jxw-beginning-prompt
      (menu-prepare-here 'jxw-dmc-wise1 nil "ꤷƲ")
    (jxw-dmc-wise1)
  )
)

(defun jxw-dmc-wise1 ()
  "1st step of delete, move, or copy"
  (interactive)
  (setq *jxw-delete-linewise-point* (point))
  (if( = *jxw-dmc-wise* 1) (beginning-of-line) )
  (setq *jxw-delete-point* (point))
  (goto-char *jxw-delete-linewise-point*)
  (menu-prepare-here 'jxw-dmc-wise2 nil "ꤷƲ")
)

(defun jxw-dmc-wise2 ()
  "2nd step of delete, move, or copy"
  (interactive)
  (if (= *jxw-dmc-wise* 1) (end-of-line) )
  (if (< (point) (point-max)) (forward-char 1) )
  (if (/= *jxw-dmc-wise* 2)
     (if (= *jxw-dmc-flag* 2)
	 (copy-region-as-kill *jxw-delete-point* (point))
       (kill-region *jxw-delete-point* (point))
     )
    (copy-rectangle-to-register ?\C-m *jxw-delete-point* (point) (/= *jxw-dmc-flag* 2))
  )
  (if (= *jxw-dmc-wise* 1) (goto-char *jxw-delete-linewise-point*) )
  (if (/= *jxw-dmc-flag* 0)
    (menu-prepare-here 'jxw-dmc-wise3 'jxw-dmc-cancel2 "ɤޤ")
  )
)

(defun jxw-dmc-cancel2 ()
  "cancel dmc-wise2"
  (interactive)
  (if (= *jxw-dmc-flag* 1) (undo 1))
)

(defun jxw-dmc-wise3 ()
  "3rd step of move and copy"
  (interactive)
  (if (= *jxw-dmc-wise* 2)
      (insert-register ?\C-m)
    (if (= *jxw-dmc-wise* 1)
	(beginning-of-line)
    )
    (yank)
  )
)

;;; End of jxw-dmc
;;; jxw-format - features for formatting documents.
(require 'popup-menu)

(defvar jxw-formatter-menu-set nil "")
(defun jxw-formatter-submenu ()
  "format the text"
  (interactive)
  (if jxw-formatter-menu-set
      ()
    (setq jxw-formatter-menu-set (make-menu-set "*եޥå*" 10))
    (menu-define-key jxw-formatter-menu-set "c" 'jxw-centering "  ")
    (menu-define-key jxw-formatter-menu-set "r" 'jxw-not-necessary "    ")
    (menu-define-key jxw-formatter-menu-set "l" 'jxw-not-necessary "    ")
    (menu-define-key jxw-formatter-menu-set "e" 'jxw-not-necessary "  ")
    (menu-define-key jxw-formatter-menu-set "n" 'jxw-not-necessary "̩  ")
    (menu-define-key jxw-formatter-menu-set "i" 'jxw-not-necessary "ǥ")
    (menu-define-key jxw-formatter-menu-set "t" 'jxw-not-necessary "      ")
    (menu-define-key jxw-formatter-menu-set "w" 'jxw-not-necessary "Ȥ    ")
    (menu-define-key jxw-formatter-menu-set "p" 'jxw-not-necessary "    ")
    (menu-define-key jxw-formatter-menu-set "k" 'jxw-not-necessary "Ȥ    ")
    (menu-define-key jxw-formatter-menu-set "s" 'jxw-not-necessary "  ")
    (menu-define-key jxw-formatter-menu-set "m" 'jxw-not-necessary "ޡ    ")
    (menu-define-key jxw-formatter-menu-set "g" 'jxw-not-necessary "      ")
    (menu-define-key jxw-formatter-menu-set "f" 'auto-fill-mode "ͤ  ")
    (menu-define-key jxw-formatter-menu-set "v" 'spell-buffer "֤ǧ  ")
  )
  (menu-display-menu jxw-formatter-menu-set)
)

(defvar *jxw-beginning-point* 0 "")
(defun jxw-centering ()
  "center text"
  (interactive)
  (menu-prepare-here 'jxw-centering1 nil "·򳫻ϤԤǲԥ򲡤Ʋ")
)

(defun jxw-centering1 ()
  ""
  (interactive)
  (setq *jxw-beginning-point* (point))
  (menu-prepare-here 'jxw-centering2 nil "ǸιԤǲԥ򲡤Ʋ")
)

(defun jxw-centering2 ()
  ""
  (interactive)
  (end-of-line)
  (center-region *jxw-beginning-point* (point))
)
;;; End od jxw-format
;;; jxw-jump - features for jumping.
(require 'popup-menu)

(defvar jxw-jump-menu-set nil "")
(defun jxw-jump-submenu ()
  "jump to the top, bottom, some line, or other window"
  (interactive)
  (if jxw-jump-menu-set
      ()
    (setq jxw-jump-menu-set (make-menu-set "**" 10))
    (menu-define-key jxw-jump-menu-set "s" 'beginning-of-buffer "ʸƬ      ")
    (menu-define-key jxw-jump-menu-set "e" 'end-of-buffer "ʸ      ")
    (menu-define-key jxw-jump-menu-set "l" 'goto-line "ֹ    ")
    (menu-define-key jxw-jump-menu-set "o" 'other-window "ɥ")
  )
  (menu-display-menu jxw-jump-menu-set)
)
;;; End of jxw-jump
;;; jxw-options - toggle flags
(provide 'jxw-options)

(defvar jxw-options-menu-set nil "Options menu")
(if jxw-options-menu-set
    ()
  (setq jxw-options-menu-set (make-menu-set "*Options*" 12))
  (menu-define-key jxw-options-menu-set "s" 'jxw-options-system "System")
  (menu-define-key jxw-options-menu-set "f" 'undefined "File    ")
  (menu-define-key jxw-options-menu-set "m" 'undefined "mode")
  (menu-define-key jxw-options-menu-set "k" 'undefined "Ķ    ")
  (menu-define-key jxw-options-menu-set "c" 'describe-bindings "Commandɽ ")
  (menu-define-key jxw-options-menu-set "x" 'jxw-options-additional "ɲõǽ¹")
)

(defun jxw-options-submenu ()
  "Options commands"
  (interactive)
  (menu-display-menu jxw-options-menu-set)
)


(defvar jxw-options-system-list nil "System options switch list")

(defun jxw-options-system ()
 "Select system options switches"
 (interactive)
 (if jxw-options-system-list
     ()
   (setq jxw-options-system-list
	 (cons "*System options List*"
	       (list
		(cons 'jxw-beginning-prompt "ϰֻ: [%c]Ϥ\t[%c]")
		(cons 'jxw-execution-prompt "¹԰ֻ: [%c]Ϥ\t[%c]")
		(cons 'make-backup-files "Хååץե: [%c]\t[%c]")
	       )
	 )
   )
 )
 (menu-switch-select jxw-options-system-list)
)

(defvar jxw-oadd-menu-set nil "Additional features; i.e. hanoi, doctor, yow, getris, etc.")
(if jxw-oadd-menu-set
    ()
  (setq jxw-oadd-menu-set (make-menu-set "*ɲõǽ*" 40))
)
;;; 
;;; (defun jxw-oadd-key (key-seq fun-name menu-item)
;;;   "Add items to jxw-oadd-menu.  \
;;; Its format is as (jxw-oadd-key \"h\" 'hanoi \"Tower of Hanoi\")"
;;;   (menu-define-key
;;;       jxw-oadd-menu-set
;;;       key-seq
;;;       (`(lambda () (interactive)
;;; 	 (autoload (quote (, fun-name)) (format "%s" (quote (, fun-name))))
;;; 	 (call-interactively (quote (, fun-name)))
;;;        ))
;;;       menu-item
;;;   )
;;; )

(defun jxw-options-additional () ""
  (interactive)
  (require 'jxwoadd)
  (menu-display-menu jxw-oadd-menu-set)
)
;;; END of jxw-options
;;; jxw-trans.el - File IO module (file transfer).

(defvar jxw-trans-menu-set nil "")

(defun jxw-trans-submenu ()
   "File-IO sub menu for jxw"
  (interactive)
  (if jxw-trans-menu-set
      ( )
    (setq jxw-trans-menu-set (make-menu-set "*file I/O*" 10))
    (menu-define-key jxw-trans-menu-set "l" 'find-file              "ɹ    ")
    (menu-define-key jxw-trans-menu-set "s" 'save-buffer   	  "¸      ")
    (menu-define-key jxw-trans-menu-set "p" 'jxw-trans-list-dir	  "      ")
    (menu-define-key jxw-trans-menu-set "i" 'jxw-trans-insert-file  "ʸ񺹹")
    (menu-define-key jxw-trans-menu-set "c" 'write-file             "ʸ̾ѹ")
    (menu-define-key jxw-trans-menu-set "f" 'jxw-not-necessary	  "ɹ")
    (menu-define-key jxw-trans-menu-set "d" 'delete-file    	  "      ")
    (menu-define-key jxw-trans-menu-set "o" 'jxw-not-necessary	  "ץ")
    (menu-define-key jxw-trans-menu-set "v" 'find-file-read-only  "ݸɹ")
    (menu-define-key jxw-trans-menu-set "b" 'jxw-format-disk	  "disk")
    (menu-define-key jxw-trans-menu-set "m" 'jxw-make-dir	 	  "dir   ")
    (menu-define-key jxw-trans-menu-set "r" 'jxw-delete-dir         "dir   ")
  )
  (menu-display-menu jxw-trans-menu-set)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun jxw-trans-list-dir ()
  "Lists directory"
  (interactive)
  (list-directory ".")
  (message "äɤ줫򲡤Ʋ")
  (read-char)
  (kill-buffer "*Directory*")
  (set-window-configuration *buffer-window-config*)
)

(defun jxw-trans-insert-file ()
  "Inserts the content of specified file after the point"
 (interactive)
 (menu-prepare-here 'insert-file 'undefined "ǲԥ򲡤Ʋ")
)

(defun jxw-delete-file (filename)
  "Deletes file for jxw"
  (interactive "fե̾?: ")
  (if (y-or-n-p
         (format "ե %s ޤOK?" filename)
      )
      (progn
	(shell-command (format "rm %s" filename))
	(message "ե %s ޤ" dirname)
      )
      (message "եκߤޤ")
  )
)

(defun jxw-format-disk ()
  "Format disk"
  (interactive)
  (message "UNIX ˾ܤͤ ޤ̤Ʋ")
)

(defun jxw-make-dir (dirname)
  "Make new directory for jxw"
  (interactive "sǥ쥯ȥ̾: ")
  (shell-command (format "mkdir %s" dirname))
  (message "ߤΥǥ쥯ȥβ %s ޤ" dirname)
)

(defun jxw-delete-dir (dirname)
  "Deletes directory recursively for jxw"
  (interactive "Dǥ쥯ȥ̾?: ")
  (if (equal (substring dirname -1) "/")
      (setq dirname (substring dirname 0 -1))
  )
  (if (y-or-n-p
         (format "ǥ쥯ȥ %s 򤽤Υե뤴ȺޤOK?" dirname)
      )
      (progn
	(shell-command (format "rm -r %s" dirname))
	(message "ǥ쥯ȥ %s 򤽤Υե뤴Ⱥޤ" dirname)
      )
      (message "ǥ쥯ȥκߤޤ")
  )
)

;;; End of trans
;;; window features

(defvar jxw-window-menu-set nil "")
(defun jxw-window-submenu ()
 ""
 (interactive)
 (if jxw-window-menu-set
     ()
   (setq jxw-window-menu-set (make-menu-set "*Window*" 12))
   (menu-define-key jxw-window-menu-set "o" 'jxw-window-open "ץ")
   (menu-define-key jxw-window-menu-set "c" 'delete-window "")
   (menu-define-key jxw-window-menu-set "b" 'switch-to-buffer "ɽʸѹ")
   (menu-define-key jxw-window-menu-set "g" 'jxw-not-necessary "̽̾ѹ")
   (menu-define-key jxw-window-menu-set "l" 'undefined "Ϣưѹ")
 )
 (menu-display-menu jxw-window-menu-set)
)

(defun jxw-window-open ()
  ""
 (interactive)
 (if jxw-execution-prompt
     (menu-prepare-here 'jxw-split-here nil "ʬ䤹ǲԥ򲡤Ʋ")
   (jxw-split-here)
 )
)

(defun jxw-split-here ()
  ""
 (interactive)
 (let ((split-point (point)) (lines 0))
   (move-to-window-line 0)
   (while (re-search-forward "\n" split-point t)
     (setq lines (1+ lines))
   )
   (split-window-vertically lines)
   (goto-char split-point)
 )
)
;;; End of window
;;  Read and send mail utilities 89-Dec-15
;;  By HASHIMOTO Ryoichi, IPRI/AIST/MITI

(require 'popup-menu)

(defvar jxw-rmail-menu-set nil "")
(defun jxw-rmail ()
  "Read mail"
  (interactive)
  (rmail)
  (if jxw-rmail-menu-set
      ()
    (setq jxw-rmail-menu-set (make-menu-set "*͹ؼ*" 10))
    (menu-define-key jxw-rmail-menu-set "n" 'rmail-next-undeleted-message "ؤ  ")
    (menu-define-key jxw-rmail-menu-set "p" 'rmail-previous-undeleted-message "ؤ  ")
    (menu-define-key jxw-rmail-menu-set "r" 'jxw-rmail-reply "ֻ")
    (menu-define-key jxw-rmail-menu-set "w" 'other-window "ֻ")
    (menu-define-key jxw-rmail-menu-set "z" 'jxw-popup-root-menu "ܥ˥")
    (menu-define-key jxw-rmail-menu-set "q" 'rmail-quit "λ  ")
    (menu-define-key jxw-rmail-menu-set "h" 'rmail-summary "ꥹ")
    (menu-define-key jxw-rmail-menu-set "d" 'rmail-delete-backward "ˤΤ  ")
  )
  (local-set-key "\e\e" 'jxw-rmail-submenu)
)

(defun jxw-rmail-submenu ()
  "displays rmail submenu"
  (interactive)
  (menu-display-menu jxw-rmail-menu-set)
)

(defun jxw-rmail-reply ()
  "reply to this mail"
  (interactive)
  (rmail-reply 0)  ; This means reply to the sender
  (jxw-set-smail-menu-set)
)

(defvar jxw-smail-menu-set nil "")
(defun jxw-set-smail-menu-set ()
  "Set send mail menu set"
  (interactive)
  (if jxw-smail-menu-set
      ()
    (setq jxw-smail-menu-set (make-menu-set "*͹*" 10))
    (menu-define-key jxw-smail-menu-set "s" 'mail-send-and-exit "ȡ      ")
    (menu-define-key jxw-smail-menu-set "a" 'kill-buffer "  ")
    (menu-define-key jxw-smail-menu-set "y" 'mail-yank-original "     ")
    (menu-define-key jxw-smail-menu-set "i" 'insert-file "file  ")
    (menu-define-key jxw-smail-menu-set "z" 'jxw-popup-root-menu "ܥ˥")
    (menu-define-key jxw-smail-menu-set "w" 'other-window "¾  ")
  (local-set-key "\e\e" 'jxw-smail-submenu)
  )
)

(defun jxw-smail ()
  "send mail sub menu"
  (interactive)
  (mail)
  (jxw-set-smail-menu-set)
)

(defun jxw-smail-submenu ()
  "Displays smail submenu"
  (interactive)
  (menu-display-menu jxw-smail-menu-set)
)

;; End of jxw-mail
